from datetime import datetime, timedelta

from netforce.access import get_active_user
from netforce.model import Model, fields, get_model

FMT_DATE="%Y-%m-%d"
FMT_DATETIME="%Y-%m-%d %H:%M:%S"

class GenVisit(Model):
    _name="clinic.gen.visit"
    _transient=True
    
    
    def _get_duration(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            duration=datetime.strptime(obj.date_to,FMT_DATETIME)-datetime.strptime(obj.date_from,FMT_DATETIME)
            res[obj.id]=duration.days
        return res
    

    _fields={
        'date_from': fields.DateTime("From", required=True),
        'date_to': fields.DateTime("To", required=True),
        'cycle_id': fields.Many2One("clinic.cycle","Cycle", required=True),
        'patient_lines': fields.One2Many("clinic.gen.visit.line","gen_id","Patient Lines"),
        'nurse_lines': fields.One2Many("clinic.gen.visit.line","gen_id","Nurse Lines"),
        'monday': fields.Boolean("Monday"),
        'tuesday': fields.Boolean("Tuesdays"),
        'wednesday': fields.Boolean("Wednesdays"),
        'thursday': fields.Boolean("Thursdays"),
        'friday': fields.Boolean("Fridays"),
        'sathurday': fields.Boolean("Sathurday"),
        'sunday': fields.Boolean("Sunday"),
        'doctor_id': fields.Many2One("clinic.doctor","Doctor"),
        'nurse_id': fields.Many2One("clinic.nurse","Nurse"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'duration': fields.Integer("Duration (hrs)", function="_get_duration"),
        "patient_type": fields.Selection([("mg","Medical Government"),("sc","Social Security"),("nhso","NHSO (30฿)"),("personal","Personal"),("others","Others")],"Patient Type"),
        'nurse_categ_id': fields.Many2One("clinic.personal.categ", "Nurse Category", domain=[['type','=','nurse']]),
    }
    

    def _get_patient_lines(self,context={}):
        ids=context.get("ids")
        if not ids:
            ids=[context.get("refer_id")]
        lines=[]
        # XXX [None]
        if any(ids):
            lines=[{'patient_id': id} for id in ids]
        return lines

    def _get_nurse(self,context={}):
        user_id=get_active_user()
        print("user_id ",user_id)
        nurse_ids=get_model("clinic.nurse").search([['user_id','=',user_id]])
        if nurse_ids:
            return nurse_ids[0]
        return None 
    
    def _get_cycle(self,context={}):
        cycle_ids=get_model('clinic.cycle').search([],order="sequence")
        if cycle_ids:
            cycle_id=cycle_ids[0]
        return cycle_id or None

    _defaults={
        'patient_lines': _get_patient_lines,
        'date_from': lambda *a: datetime.now().strftime(FMT_DATETIME),
        'date_to': lambda *a: (datetime.now()+timedelta(days=7)).strftime(FMT_DATETIME),
        'nurse_id': _get_nurse,
        'duration': 1,
        'cycle_id': _get_cycle,
        'monday': True,
    }

    def onchange_time(self,context={}):
        data=context['data']
        cycle_id=data['cycle_id']
        duration=1
        if cycle_id:
            cycle=get_model('clinic.cycle').browse(cycle_id)
            duration=cycle.duration or 0
        seconds=duration*3600
        date_from=data['date_from']
        date_to=(datetime.strptime(date_from,FMT_DATETIME)+timedelta(seconds=seconds)).strftime(FMT_DATETIME)
        data['date_to']='%s %s'%(data['date_to'][0:10],date_to[11:])
        duration=(datetime.strptime(data['date_to'],FMT_DATETIME)-datetime.strptime(data['date_from'],FMT_DATETIME)).seconds/3600
        data['duration']=duration
        return data

    def onchange_dateto(self,context={}):
        data=context['data']
        data['duration']=(datetime.strptime(data['date_to'],FMT_DATETIME)-datetime.strptime(data['date_from'],FMT_DATETIME)).seconds/3600
        return data

    def gen_visit(self,ids,context):
        obj=self.browse(ids)[0]
        visit_obj=get_model('clinic.visit')
        days=[
            obj.monday and 1 or 0,
            obj.tuesday and 2 or 0,
            obj.wednesday and 3 or 0,
            obj.thursday and 4 or 0,
            obj.friday and 5 or 0,
            obj.sathurday and 6 or 0,
            obj.sunday and 7 or 0,
            ]
        days=[day for day in days if day]
        if not days:
            raise Exception("Please select Day")
        visit_vals=[]
        date_from=datetime.strptime(obj.date_from,FMT_DATETIME)
        date_to=datetime.strptime(obj.date_to,FMT_DATETIME)
        patients=[p.patient_id.id for p in obj.patient_lines if p.patient_id]
        if not patients and obj.patient_type:
            patients=get_model("clinic.patient").search([['type','=',obj.patient_type]])
        elif patients:
            # continue to generate with condition
            pass
        else:
            raise Exception("Please select some patient or patient type")

        schedules={}
        for patient_id in patients:
            ntoday=1
            day_total=(date_to-date_from).days+ntoday
            for weekday in days:
                wd=date_from.weekday() 
                start_date=date_from
                while wd != weekday-1:
                    start_date+=timedelta(days=1)
                    wd=start_date.weekday()

                #XXX should gen only in scope
                if start_date.strftime(FMT_DATE) > date_to.strftime(FMT_DATE):
                    continue
                count=0
                while count < day_total:
                    tmp=start_date+timedelta(days=count)
                    ttime_start="%s %s" % (tmp.strftime(FMT_DATE),date_from.strftime(FMT_DATETIME)[11:])
                    ttime_stop="%s %s" % (tmp.strftime(FMT_DATE),date_to.strftime(FMT_DATETIME)[11:])
                    vals={
                        'patient_id': patient_id,
                        'doctor_id': obj.doctor_id.id,
                        'nurse_id': obj.nurse_id.id,
                        'department_id': obj.department_id.id,
                        'cycle_id': obj.cycle_id.id,
                        'time_start': ttime_start,
                        'time_stop': ttime_stop,
                        'state': 'draft',
                    }
                    visit_vals.append(vals)
                    count+=7

                    dom=[]
                    dom.append(['time_start','>=','%s %s'%(tmp.strftime(FMT_DATE)[0:10],' 00:00:00')])
                    dom.append(['time_stop','<=','%s %s'%(tmp.strftime(FMT_DATE)[0:10],' 23:59:59')])
                    dom.append(['patient_id', '=', patient_id])
                    dom.append(['cycle_id', '=', obj.cycle_id.id]) #XXX
                    dom.append(['state','=','draft'])
                    vids=visit_obj.search(dom)
                    visit_obj.delete(vids)
                    key='%s-%s'%(vals['cycle_id'], vals['time_start'])
                    if not schedules.get(key):
                        schedules[key]={
                            'cycle_id': obj.cycle_id.id,
                            'time_start': ttime_start,
                            'time_stop': ttime_stop,
                            'state':'draft',
                        }

                start_date=tmp

        for vals in visit_vals:
            visit_obj.create(vals)

        nurse_ids=[]
        for nurse_line in obj.nurse_lines:
            nurse_id=nurse_line.nurse_id.id
            # None
            if nurse_id:
                nurse_ids.append(nurse_id)
        if not nurse_ids:
            nurse_ids=get_model("clinic.personal").search([['type','=','nurse'],['categ_id','=',obj.nurse_categ_id.id]])

        dom=[]
        dom.append(['time_start','>=','%s %s'%(obj.date_from[0:10],' 00:00:00')])
        dom.append(['time_stop','<=','%s %s'%(start_date.strftime(FMT_DATE)[0:10],' 23:59:59')])
        dom.append(['cycle_id','=',obj.cycle_id.id])
        schedule_obj=get_model("clinic.schedule")
        schedule_ids=schedule_obj.search(dom)
        schedule_obj.delete(schedule_ids)
        for key, vals in schedules.items():
            vals['nurses']=[('add', nurse_ids)]
            schedule_obj.create(vals)
            
        return {
            'next': {
                'name': 'clinic_visit',
                'mode': 'list',
            },
            'flash': 'Generate OK',
        }

    def clear_visit(self,ids,context={}):
        obj=self.browse(ids)[0]
        days=[
            obj.monday and 1 or 0,
            obj.tuesday and 2 or 0,
            obj.wednesday and 3 or 0,
            obj.thursday and 4 or 0,
            obj.friday and 5 or 0,
            ]
        days=[day for day in days if day]
        visit_ids=[]
        date_from=datetime.strptime(obj.date_from,FMT_DATETIME)
        date_to=datetime.strptime(obj.date_to,FMT_DATETIME)
        # FIXME add more 1 week for make sure
        date_to2=(date_to+timedelta(days=7)).strftime(FMT_DATETIME)
        visit_obj=get_model("clinic.visit")
        schedule_obj=get_model("clinic.schedule")

        patients=[p.patient_id.id for p in obj.patient_lines if p.patient_id]
        if not patients and obj.patient_type:
            patients=get_model("clinic.patient").search([['type','=',obj.patient_type]])
        elif patients:
            # continue to create with critiria
            pass
        else:
            dom=[]
            dom.append(['time_start','>=','%s %s'%(obj.date_from[0:10],' 00:00:00')])
            dom.append(['time_stop','<=','%s %s'%(date_to2[0:10],' 23:59:59')])
            dom.append(['cycle_id','=',obj.cycle_id.id])
            dom.append(['state','=','draft'])
            vids=visit_obj.search(dom)
            visit_obj.delete(vids)
            schedule_obj=get_model("clinic.schedule")
            schedule_ids=schedule_obj.search(dom)
            schedule_obj.delete(schedule_ids)
            return {
                'next': {
                    'name': 'clinic_gen_visit_form',
                    'mode': 'form',
                },
                'flash': 'Clear OK',
            }

        if days:
            for patient_id in patients:
                # loop days in weekend
                ntoday=1
                day_total=(date_to-date_from).days+ntoday
                for weekday in days:
                    wd=date_from.weekday() 
                    start_date=date_from
                    while wd != weekday-1:
                        start_date+=timedelta(days=1)
                        wd=start_date.weekday()

                    #XXX should gen only in scop
                    if start_date.strftime(FMT_DATE) > date_to.strftime(FMT_DATE):
                        continue
                    count=0
                    while count < day_total:
                        tmp=start_date+timedelta(days=count)
                        time_start="%s 00:00:00" % (tmp.strftime(FMT_DATE))
                        time_stop="%s 23:59:59" % (tmp.strftime(FMT_DATE))
                        dom=[]
                        dom.append(['time_start','>=','%s'%time_start])
                        dom.append(['time_stop','<=','%s'%time_stop])
                        dom.append(['patient_id','=','%s'%patient_id])
                        dom.append(['state','=','draft'])
                        count+=7
                        vids=visit_obj.search(dom)
                        if vids:
                            visit_ids.append(vids[0])
                    start_date=tmp
            else:
                time_start=obj.date_from
                time_stop=obj.date_to
                for patient_id in patients:
                    dom=[]
                    dom.append(['time_start','>=','%s'%time_start])
                    dom.append(['time_stop','<=','%s'%time_stop])
                    dom.append(['patient_id','=','%s'%patient_id])
                    dom.append(['state','=','draft'])
                    vids=visit_obj.search(dom)
                    if vids:
                        visit_ids.append(vids[0])
        if visit_ids:
            get_model("clinic.visit").delete(visit_ids)
        dom=[]
        dom.append(['time_start','>=','%s %s'%(obj.date_from[0:10],' 00:00:00')])
        dom.append(['time_stop','<=','%s %s'%(date_to2[0:10],' 23:59:59')])
        dom.append(['cycle_id','=',obj.cycle_id.id])
        schedule_ids=schedule_obj.search(dom)
        schedule_obj.delete(schedule_ids)
        return {
            'next': {
                'name': 'clinic_visit',
                'mode': 'list',
            },
            'flash': 'Clear OK',
        }

GenVisit.register()

